@@ -6,6 +6,8 @@ use Brackup::ChunkIterator;
 use Brackup::CompositeChunk;
 use Brackup::GPGProcManager;
 use Brackup::GPGProcess;
+use File::Basename;
+use File::Temp qw(tempfile);
 
 sub new {
     my ($class, %opts) = @_;
@@ -15,11 +17,18 @@ sub new {
     $self->{target}  = delete $opts{target};   # Brackup::Target
     $self->{dryrun}  = delete $opts{dryrun};   # bool
     $self->{verbose} = delete $opts{verbose};  # bool
+    $self->{inventory} = delete $opts{inventory};  # bool
+    $self->{savefiles} = delete $opts{savefiles};  # bool
     $self->{zenityprogress} = delete $opts{zenityprogress};  # bool
 
     $self->{modecounts} = {}; # type -> mode(octal) -> count
+    $self->{idcounts}   = {}; # type -> uid/gid -> count
+
+    $self->{_uid_map} = {};   # uid -> username
+    $self->{_gid_map} = {};   # gid -> group
 
     $self->{saved_files} = [];   # list of Brackup::File objects backed up
+    $self->{unflushed_files} = [];   # list of Brackup::File objects not in backup_file
 
     croak("Unknown options: " . join(', ', keys %opts)) if %opts;
 
@@ -35,7 +44,7 @@ sub backup {
 
     my $stats  = Brackup::BackupStats->new;
 
-    my $gpg_rcpt = $self->{root}->gpg_rcpt;
+    my @gpg_rcpts = $self->{root}->gpg_rcpts;
 
     my $n_kb         = 0.0; # num:  kb of all files in root
     my $n_files      = 0;   # int:  # of files in root
@@ -43,6 +52,7 @@ sub backup {
 
     # if we're pre-calculating the amount of data we'll
     # actually need to upload, store it here.
+    my $n_files_up   = 0;
     my $n_kb_up      = 0.0;
     my $n_kb_up_need = 0.0; # by default, not calculated/used.
 
@@ -54,11 +64,15 @@ sub backup {
     $root->foreach_file(sub {
         my ($file) = @_;  # a Brackup::File
         push @files, $file;
+        $self->record_mode_ids($file);
         $n_files++;
         $n_kb += $file->size / 1024;
     });
 
     $self->debug("Number of files: $n_files\n");
+    $stats->timestamp('File Discovery');
+    $stats->set('Number of Files' => $n_files);
+    $stats->set('Total File Size' => sprintf('%0.01f MB', $n_kb / 1024));
 
     # calc needed chunks
     if ($ENV{CALC_NEEDED}) {
@@ -76,25 +90,55 @@ sub backup {
             }
         }
         warn "kb need to upload = $n_kb_up_need\n";
+        $stats->timestamp('Calc Needed');
     }
 
 
     my $chunk_iterator = Brackup::ChunkIterator->new(@files);
+    undef @files;
+    $stats->timestamp('Chunk Iterator');
 
     my $gpg_iter;
     my $gpg_pm;   # gpg ProcessManager
-    if ($gpg_rcpt) {
+    if (@gpg_rcpts) {
         ($chunk_iterator, $gpg_iter) = $chunk_iterator->mux_into(2);
         $gpg_pm = Brackup::GPGProcManager->new($gpg_iter, $target);
     }
 
+    # begin temp backup_file
+    my ($metafh, $meta_filename);
+    unless ($self->{dryrun}) {
+        ($metafh, $meta_filename) = tempfile(
+                                             '.' . basename($backup_file) . 'XXXXX',
+                                             DIR => dirname($backup_file),
+        );
+        if (! @gpg_rcpts) {
+            if (eval { require IO::Compress::Gzip }) {
+                close $metafh;
+                $metafh = IO::Compress::Gzip->new($meta_filename)
+                    or die "Cannot open tempfile with IO::Compress::Gzip: $IO::Compress::Gzip::GzipError";
+            }
+        }
+        print $metafh $self->backup_header;
+    }
+
     my $cur_file; # current (last seen) file
     my @stored_chunks;
     my $file_has_shown_status = 0;
 
+    my $merge_under = $root->merge_files_under;
+    my $comp_chunk  = undef;
+
     my $end_file = sub {
         return unless $cur_file;
-        $self->add_file($cur_file, [ @stored_chunks ]);
+        if ($merge_under && $comp_chunk) {
+            # defer recording to backup_file until CompositeChunk finalization
+            $self->add_unflushed_file($cur_file, [ @stored_chunks ]);
+        }
+        else {
+            print $metafh $cur_file->as_rfc822([ @stored_chunks ], $self) if $metafh;
+        }
+        $self->add_saved_file($cur_file, [ @stored_chunks ]) if $self->{savefiles};
         $n_files_done++;
         $n_kb_done += $cur_file->size / 1024;
         $cur_file = undef;
@@ -129,9 +173,6 @@ sub backup {
         $file_has_shown_status = 0;
     };
 
-    my $merge_under = $root->merge_files_under;
-    my $comp_chunk  = undef;
-
     # records are either Brackup::File (for symlinks, directories, etc), or
     # PositionedChunks, in which case the file can asked of the chunk
     while (my $rec = $chunk_iterator->next) {
@@ -164,7 +205,10 @@ sub backup {
             next;
         }
 
-        $show_status->() unless $file_has_shown_status++;
+        unless ($file_has_shown_status++) {
+            $show_status->();
+            $n_files_up++;
+        }
         $self->debug("  * storing chunk: ", $pchunk->as_string, "\n");
         $self->report_progress(undef, $pchunk->file->path . " (" . $pchunk->offset . "," . $pchunk->length . ")");
 
@@ -172,7 +216,7 @@ sub backup {
             $schunk = Brackup::StoredChunk->new($pchunk);
 
             # encrypt it
-            if ($gpg_rcpt) {
+            if (@gpg_rcpts) {
                 $schunk->set_encrypted_chunkref($gpg_pm->enc_chunkref_of($pchunk));
             }
 
@@ -192,6 +236,7 @@ sub backup {
                     $self->debug("Finalizing composite chunk $comp_chunk...");
                     $comp_chunk->finalize;
                     $comp_chunk = undef;
+                    $self->flush_files($metafh);
                 }
                 $comp_chunk ||= Brackup::CompositeChunk->new($root, $target);
                 $comp_chunk->append_little_chunk($schunk);
@@ -211,6 +256,7 @@ sub backup {
 
 
             $n_kb_up += $pchunk->length / 1024;
+            $schunk->forget_chunkref;
             push @stored_chunks, $schunk;
         }
 
@@ -227,55 +273,63 @@ sub backup {
             #}
         }
 
+        $stats->check_maxmem;
         $pchunk->forget_chunkref;
-        $schunk->forget_chunkref if $schunk;
     }
     $end_file->();
     $comp_chunk->finalize if $comp_chunk;
+    $self->flush_files($metafh);
+    $stats->timestamp('Chunk Storage');
+    $stats->set('Number of Files Uploaded:', $n_files_up);
+    $stats->set('Total File Size Uploaded:', sprintf('%0.01f MB', $n_kb_up / 1024));
 
     unless ($self->{dryrun}) {
-        # write the metafile
-        $self->debug("Writing metafile ($backup_file)");
-        $self->report_progress(100, "Saving metafile " . $backup_file);
-        open (my $metafh, ">$backup_file") or die "Failed to open $backup_file for writing: $!\n";
-        print $metafh $self->backup_header;
-        $self->foreach_saved_file(sub {
-            my ($file, $schunk_list) = @_;
-            print $metafh $file->as_rfc822($schunk_list, $self);  # arrayref of StoredChunks
-        });
-        close $metafh or die;
+        close $metafh or die "Close on metafile '$backup_file' failed: $!";
+        rename $meta_filename, $backup_file
+            or die "Failed to rename temporary backup_file: $!\n";
 
-        my $contents;
+        my ($store_fh, $store_filename);
+        my $is_encrypted = 0;
 
         # store the metafile, encrypted, on the target
-        if ($gpg_rcpt) {
+        if (@gpg_rcpts) {
             my $encfile = $backup_file . ".enc";
+            my @recipients = map {("--recipient", $_)} @gpg_rcpts;
             system($self->{root}->gpg_path, $self->{root}->gpg_args,
+                   @recipients,
                    "--trust-model=always",
-                   "--recipient", $gpg_rcpt, "--encrypt", "--output=$encfile", "--yes", $backup_file)
+                   "--batch",
+                   "--encrypt", 
+                   "--output=$encfile", 
+                   "--yes", 
+                   $backup_file)
                 and die "Failed to run gpg while encryping metafile: $!\n";
-            $contents = _contents_of($encfile);
-            unlink $encfile;
+            open ($store_fh, $encfile) or die "Failed to open encrypted metafile '$encfile': $!\n";
+            $store_filename = $encfile;
+            $is_encrypted = 1;
         } else {
-            $contents = _contents_of($backup_file);
+            # Reopen $metafh to reset file pointer (no backward seek with IO::Compress::Gzip)
+            open($store_fh, $backup_file) or die "Failed to open metafile '$backup_file': $!\n";
+            $store_filename = $backup_file;
         }
 
         # store it on the target
         $self->debug("Storing metafile to " . ref($target));
         my $name = $self->{root}->publicname . "-" . $self->backup_time;
-        $target->store_backup_meta($name, $contents);
+        $target->store_backup_meta($name, $store_fh, { filename => $store_filename, is_encrypted => $is_encrypted });
+        $stats->timestamp('Metafile Storage');
+
+        # cleanup encrypted metafile
+        if ($is_encrypted) {
+            close $store_fh or die "Close on encrypted metafile failed: $!";
+            unlink $store_filename;
+        }
     }
     $self->report_progress(100, "Backup complete.");
 
     return $stats;
 }
 
-sub _contents_of {
-    my $file = shift;
-    open (my $fh, $file) or die "Failed to read contents of $file: $!\n";
-    return do { local $/; <$fh>; };
-}
-
 sub default_file_mode {
     my $self = shift;
     return $self->{_def_file_mode} ||= $self->_default_mode('f');
@@ -292,6 +346,48 @@ sub _default_mode {
     return (sort { $map->{$b} <=> $map->{$a} } keys %$map)[0];
 }
 
+sub default_uid {
+    my $self = shift;
+    return $self->{_def_uid} ||= $self->_default_id('u');
+}
+
+sub default_gid {
+    my $self = shift;
+    return $self->{_def_gid} ||= $self->_default_id('g');
+}
+
+sub _default_id {
+    my ($self, $type) = @_;
+    my $map = $self->{idcounts}{$type} || {};
+    return (sort { $map->{$b} <=> $map->{$a} } keys %$map)[0];
+}
+
+# space-separated list of local uid:username mappings
+sub uid_map {
+    my $self = shift;
+    my @map;
+    my $uidcounts = $self->{idcounts}{u};
+    for my $uid (sort { $a <=> $b } keys %$uidcounts) {
+      if (my $name = getpwuid($uid)) {
+        push @map, "$uid:$name";
+      }
+    }
+    return join(' ', @map);
+}
+
+# space-separated list of local gid:group mappings
+sub gid_map {
+    my $self = shift;
+    my @map;
+    my $gidcounts = $self->{idcounts}{g};
+    for my $gid (sort { $a <=> $b } keys %$gidcounts) {
+      if (my $name = getgrgid($gid)) {
+        push @map, "$gid:$name";
+      }
+    }
+    return join(' ', @map);
+}
+
 sub backup_time {
     my $self = shift;
     return $self->{backup_time} ||= time();
@@ -304,30 +400,54 @@ sub backup_header {
     $ret .= "BackupTime: " . $now . " (" . localtime($now) . ")\n";
     $ret .= "BackupDriver: " . ref($self->{target}) . "\n";
     if (my $fields = $self->{target}->backup_header) {
-        foreach my $k (keys %$fields) {
+        foreach my $k (sort keys %$fields) {
             die "Bogus header field from driver" unless $k =~ /^\w+$/;
             my $val = $fields->{$k};
+            next if ! defined $val || $val eq '';   # skip keys with empty values
             die "Bogus header value from driver" if $val =~ /[\r\n]/;
             $ret .= "Driver-$k: $val\n";
         }
     }
     $ret .= "RootName: " . $self->{root}->name . "\n";
     $ret .= "RootPath: " . $self->{root}->path . "\n";
+    $ret .= "TargetName: " . $self->{target}->name . "\n";
     $ret .= "DefaultFileMode: " . $self->default_file_mode . "\n";
     $ret .= "DefaultDirMode: " . $self->default_directory_mode . "\n";
-    if (my $rcpt = $self->{root}->gpg_rcpt) {
-        $ret .= "GPG-Recipient: $rcpt\n";
-    }
+    $ret .= "DefaultUID: " . $self->default_uid . "\n";
+    $ret .= "DefaultGID: " . $self->default_gid . "\n";
+    $ret .= "UIDMap: " . $self->uid_map . "\n";
+    $ret .= "GIDMap: " . $self->gid_map . "\n";
+    $ret .= "GPG-Recipient: $_\n" for $self->{root}->gpg_rcpts;
     $ret .= "\n";
     return $ret;
 }
 
-sub add_file {
-    my ($self, $file, $handlelist) = @_;
+sub record_mode_ids {
+    my ($self, $file) = @_;
     $self->{modecounts}{$file->type}{$file->mode}++;
-    push @{ $self->{saved_files} }, [ $file, $handlelist ];
+    $self->{idcounts}{u}{$file->uid}++;
+    $self->{idcounts}{g}{$file->gid}++;
 }
 
+sub add_unflushed_file {
+    my ($self, $file, $handlelist) = @_;
+    push @{ $self->{unflushed_files} }, [ $file, $handlelist ];
+}   
+
+sub flush_files {
+    my ($self, $fh) = @_;
+    while (my $rec = shift @{ $self->{unflushed_files} }) {
+      next unless $fh;
+      my ($file, $stored_chunks) = @$rec;
+      print $fh $file->as_rfc822($stored_chunks, $self);
+    }
+}
+
+sub add_saved_file {
+    my ($self, $file, $handlelist) = @_;
+    push @{ $self->{saved_files} }, [ $file, $handlelist ];
+}   
+
 sub foreach_saved_file {
     my ($self, $cb) = @_;
     foreach my $rec (@{ $self->{saved_files} }) {
